/******************************************************************************
 *
 * FILE: can.h
 *
 * Description:
 *    CAN communication
 *
 * History:
 *    17.11.2017   PN   Initial version
 *    13.02.2018   MF   Reuse for SPC574K72
 *
 *
 ******************************************************************************
 * Copyright (C) 2015-2017 HighTec EDV-Systeme GmbH.  All rights reserved.
 *
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/


#ifndef CAN_H
#define CAN_H

/* ============================================================================
 * CAN EXAMPLE SETTING
 * User ** can ** change it, here in this file
 * ==========================================================================*/

/* External Loop Back mode enable(1) or disable(0)
 * See RM Chapter - External Loop Back mode */
#define CAN_LOOP_BACK_TEST    0

/* ============================================================================
 * CAN, PIN INSTANCES IMPLEMENTATION
 * Only relevant configuration for CAN example
 * ==========================================================================*/
/* CAN Protocol clock*/
#define CAN_PROTOCOL_CLOCK UC_CAN_CLK0  /* The clock is set via BSP during
                                           platform initialization.*/

/* CAN Baud rate */

/* CAN_PROTOCOL_CLOCK /((TSEG1 + TSEG2 + 3) * BRP) = 500 kbit/s (default value
 * for CAN EXAMPLE )*/

/* The default CAN EXAMPLE configuration assume
 * CAN_PROTOCOL_CLOCK/(BRP * CCCU.CCFG.) = 10MHz */

#define CAN_PROTOCOL_CLK_DIV      (CAN_PROTOCOL_CLOCK/(10))

#define CAN_PROTOCOL_CLK_DIV_NOT_VALID  \
	((CAN_PROTOCOL_CLK_DIV * (10))- CAN_PROTOCOL_CLOCK)

#if ((CAN_PROTOCOL_CLOCK > 10) && !CAN_PROTOCOL_CLK_DIV_NOT_VALID)
#define CAN_BTP_BRP               (CAN_PROTOCOL_CLK_DIV - 1)
#define CAN_BTP_TSEG1    14
#define CAN_BTP_TSEG2    3
#define CAN_BTP_SJW      0
#else
#error The protocol frequency is not supported
#endif

/* Shared CAN Message RAM */
#define CAN_SHARED_RAM_ADD_START      0xF7ED4000UL
#define CAN_SHARED_RAM_ADD_END        0xF7ED7FFFUL

/* FIFO offsets for Shared memory */
#define CAN_TX_FIFO_OFFSET            0x0
#define CAN_RX_FIFO_OFFSET            0x100
#define CAN_STD_FILTER_OFFSET         0x200

/* CAN port alternate functions */
#define CAN_PIN_SSS_TX 		    0x1  /* Port pin functionality */
#define CAN_PIN_SSS_RX 		    0x2  /* SIUL2_MSCR SSS field */

/* CAN ports */
#define CAN_PIN_PA10		10
#define CAN_PIN_PA11	    11
#define CAN_PIN_MUX_PA11    246 /* MUX MSCR_MUX[758] NOTE:(758-512 = 246) */

/* CAN Data received ISR Vector */
#define UC_CAN_ISR_INT0 688    /* M_CAN_1 Line0 combined interrupt */

/* ============================================================================
 * CAN API INTERFACE
 * ==========================================================================*/

void can_Init(void);
void can_SendData(void);

/* ============================================================================
 * CAN TYPES
 * ==========================================================================*/
typedef struct
{
    uint32_t    ID;
    uint8_t     XTD;
    uint8_t     EDL;
    uint8_t     BRS;
    uint8_t     DLC;
    uint8_t     Data[8];
}
CAN_MESSAGE_OBJECT_t;

/* Structures for FIFO elements */
/* Receive FIFO element */
typedef union {
    struct{
       uint32_t R0;
       uint32_t R1;
       uint32_t R2;
       uint32_t R3;
    }R;
    struct{
       //R0
       uint32_t ESI: 		1;  /* Transmitting node is error active/passive */
       uint32_t XTD:      	1;	/* Extended Identifier */
       uint32_t RTR:      	1;	/* Remote Transmission Request */
       uint32_t ID:      	29; /* Standard or extended identifier */
       //R1
       uint32_t ANMF:     	1;	/* Accepted Non-matching Frame */
       uint32_t FIDX:     	7;	/* Filter index */
       uint32_t res0:		2;
       uint32_t EDL:		1;  /* Standard/CAN FD frame format*/
       uint32_t BRS:		1;  /* Bit Rate Switch */
       uint32_t DLC:      	4;  /* Data Length Code */
       uint32_t RXTS:       16; /* Timestamp Counter value */
       //R2
       uint8_t DB3;
       uint8_t DB2;
       uint8_t DB1;
       uint8_t DB0;
       //R3
       uint8_t DB7;
       uint8_t DB6;
       uint8_t DB5;
       uint8_t DB4;
    }B;
}tCAN_RX_FIFO_ELEMENT;

/* Transmit buffer FIFO element */
typedef union {
	   struct{
		   vuint32_t T0;
		   vuint32_t T1;
		   vuint32_t T2;
		   vuint32_t T3;
	   }R;
       struct{
			//T0
			vuint32_t ESI:      1;  /* Error State Indicator */
			vuint32_t XTD:      1;	/* Extended Identifier */
			vuint32_t RTR:      1;	/* Remote Transmission Request */
			vuint32_t ID:      29;	/* ID */
			//T1
			vuint32_t  MM:      8;	/* Message Marker (written by CPU) */
			vuint32_t  ET:      2;	/* Event FIFO Control (don't store/store)*/
			vuint32_t  EDL:     1;  /* Extended Data Length*/
			vuint32_t  BRS:     1;  /* Bit Rate Switch*/
			vuint32_t  DLC:     4;	/* Data Length Code */
			vuint32_t  TXTS:    16; /* Tx Timestamp */
			//T2
			vuint8_t  DB3;
			vuint8_t  DB2;
			vuint8_t  DB1;
			vuint8_t  DB0;
			//T3
			vuint8_t  DB7;
			vuint8_t  DB6;
			vuint8_t  DB5;
			vuint8_t  DB4;
       }B;
} tCAN_TX_FIFO_ELEMENT;

/* ============================================================================
 * CAN EXTERN
 * ==========================================================================*/

extern CAN_MESSAGE_OBJECT_t g_can_TxData;
extern CAN_MESSAGE_OBJECT_t g_can_RxData;

/* ============================================================================
 * CAN CONFIGURATION
 * Example setting
 * ==========================================================================*/

#define CAN_BTP_CFG (((CAN_BTP_BRP)<<16)|((CAN_BTP_TSEG1)<<8)| \
		             ((CAN_BTP_TSEG2)<<0)|((CAN_BTP_SJW)<<25))

#define CAN_TX_FIFO_RAM (*(volatile tCAN_TX_FIFO_ELEMENT *) \
		                  (CAN_SHARED_RAM_ADD_START + CAN_TX_FIFO_OFFSET))

#define CAN_RX_FIFO_RAM (*(volatile tCAN_RX_FIFO_ELEMENT *) \
		                  (CAN_SHARED_RAM_ADD_START + CAN_RX_FIFO_OFFSET))

/* ============================================================================
 * CAN CONFIGURATION CHECKS
 * ==========================================================================*/

#endif /* CAN_H */
