/******************************************************************************
 * FILE:
 *    uc_spc58nn84_regdefs.h
 *
 * UC ARCHITECTURE:
 *    PowerPC
 *
 * UC FAMILY:
 *    SPC58
 *
 * DESCRIPTION:
 *    Goals
 *       1. It provides certain independence on peripheral header files
 *          from the vendor that might change from one version to another
 *          and between derivatives.
 *       2. Only those used by BSP API functions are defined. The set shall be
 *          kept on minimum to have high probability that it will works across
 *          different derivatives where changes in the module IP implementation
 *          are not expected.
 *       3. The only dependence on target register header file is due to
 *          necessity of module Base address knowledge.
 *
 *    Register definition and access
 *       X_REG
 *          represents register that can be used to read or write register
 *          value directly.
 *          Example: X_REG = VAL
 *       X_REG_BASE
 *          address of the first register from the set, where the other
 *          registers are placed with a known offset => can be calculated.
 *          The internal structure of the same HW module is assumed
 *          to be kept the same among uC derivatives
 *          Example: *(X_REG_BASE + index) = VAL
 *       X_REG_OFFSET
 *          Indirect access through Base address and correct typecast
 *          X_REG_OFFSET must correspond to the typecast size
 *          Example: *(vuintX_t*)(BASE + X_REG_OFFSET) = VAL
 *
 * FILE HISTORY:
 *    V1.0    23.09.2017     RO
 *       > Initial version
 *    V2.0    25.01.2018     RO
 *       > File name rename and file header update
 *
 ******************************************************************************
 * Copyright (C) 2015-2018 HighTec EDV-Systeme GmbH.  All rights reserved.
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/

#ifndef UC_SPC58NN84_REGDEFS_H
#define UC_SPC58NN84_REGDEFS_H


/* ============================================================================
 * SPC MODULE BASE ADDRESSES AND REGISTERS USED BY BSP LLD
 * ==========================================================================*/

/* ----------------------------------------------------------------------------
 * MODE ENTRY module
 * --------------------------------------------------------------------------*/

/* Registers */
#define UC_ME_GS_REG              *(vuint32_t *) (UC_ME_BASE + 0x0000)
#define UC_ME_MCTL_REG            *(vuint32_t *) (UC_ME_BASE + 0x0004)
#define UC_ME_DRUN_MC_REG         *(vuint32_t *) (UC_ME_BASE + 0x002C)
#define UC_ME_RUN_PC0_REG         *(vuint32_t *) (UC_ME_BASE + 0x0080)
#define UC_ME_CCTL_BASE            (vuint16_t *) (UC_ME_BASE + 0x01C4) /* ME_CCTL0 */
#define UC_ME_CADDR_BASE           (vuint32_t *) (UC_ME_BASE + 0x01E0) /* ME_CADDR0 */

/* Bits definition */
#define UC_ME_CCTL_DRUN_BIT        (uint16_t) 0x0008
#define UC_ME_CCTL_ALL_RUN_BIT     (uint16_t) 0x00F8
#define UC_ME_RUNPC_DRUN_BIT       (uint32_t) 0x00000008
#define UC_ME_GS_MTRANS_BIT        (uint32_t) 0x08000000
#define UC_ME_MODE_PLL0_BIT        (uint32_t) 0x00000040
#define UC_ME_MODE_PLL1_BIT        (uint32_t) 0x00000080
#define UC_ME_MODE_XOSC_BIT        (uint32_t) 0x00000020

/* Gets MODE value from GS status register */
#define MC_ME_GS_GET_MODE(val) \
(uint32_t)(((val) & (uint32_t) 0xF0000000) >> 28)

/* Sets SYSCLK value in one of the mode MC registers */
#define UC_ME_SET_SYSCLK(reg,val) \
((reg) |= (((reg) & 0xFFFFFFF0) | ((val) & 0x0000000F)))

/* Gets SYSCLK value from GS status register */
#define UC_ME_GS_GET_SYSCLK(val) \
(uint32_t)((val) & 0x0000000F)

/* ----------------------------------------------------------------------------
 * CGM module
 * --------------------------------------------------------------------------*/

/* Registers */
#define UC_CGM_SC_DC_REG_BASE      (vuint32_t *) (UC_CGM + 0x01E8) /* CGM_SC_DC0 */
#define UC_CGM_AC_REG_BASE         (vuint32_t *) (UC_CGM + 0x0200) /* CGM_AC0_SC */
#define UC_CGM_AC_DC_REG_BASE      (vuint32_t *) (UC_CGM + 0x0208) /* CGM_AC0_SC */
#define UC_CGM_AC_OFFSET           (uint32_t)    0x20  /* AC(x)_SC -> AC(x+1)_SC */
#define UC_CGM_AC_DC_OFFSET        (uint32_t)    0x08  /* AC(x)_SC -> AC(x)_DC0 */
#define UC_CGM_AC_DC_DC_OFFSET     (uint32_t)    0x04  /* AC(x)_DC(x) -> AX(x)_DC(x+1) */

/* CGM AUX AC register address */
#define UC_CGM_AC_REG_ADDR(aux) \
((uint32_t)UC_CGM_AC_REG_BASE + (aux * UC_CGM_AC_OFFSET))

/* CGM AUX DC register address */
#define UC_CGM_AC_DC_REG_ADDR(aux,dc) \
(UC_CGM_AC_REG_ADDR(aux) + UC_CGM_AC_DC_OFFSET + (dc * UC_CGM_AC_DC_DC_OFFSET))

/* CGM AUX AC register */
#define UC_CGM_AC_REG(aux)        *(vuint32_t *) (UC_CGM_AC_REG_ADDR(aux))

/* CGM AUX DC register */
#define UC_CGM_AC_DC_REG(aux,dc)  *(vuint32_t *) (UC_CGM_AC_DC_REG_ADDR(aux,dc))

/* Sets SELCTL value */
#define UC_CGM_AC_SET_SELCTL_VAL(sel) \
(uint32_t)(((sel) & (uint32_t)0xF) << 24)

/* Sets DC register with divider values
 * div == 0 : disable divider (clock)
 * div == N : divide clock by N */
#define UC_CGM_SET_DC_VAL(div) \
(uint32_t)(((div) == 0) ? 0 : (0x80000000 | ((((div)-1) & 0x00007FFF) << 16)))

/* ----------------------------------------------------------------------------
 * PLL module
 * --------------------------------------------------------------------------*/

/* Registers */
#define UC_PLL0_DV_REG            *(vuint32_t *) (UC_PLL_BASE + 0x0008)
#define UC_PLL1_DV_REG            *(vuint32_t *) (UC_PLL_BASE + 0x0028)
#define UC_PLL0_DV_REG_BASE        (vuint32_t *) (UC_PLL_BASE + 0x0008)
#define UC_PLL1_DV_REG_BASE        (vuint32_t *) (UC_PLL_BASE + 0x0028)

/* Prepare value for PLL0 DV register */
#define UC_PLL0_DV_VAL(prediv, mfd, rfdphi, rfdphi1) \
(uint32_t)(((rfdphi1 & 0x0F) << 27) |  \
           ((rfdphi  & 0x3F) << 16) |  \
           ((prediv  & 0x07) << 12) |  \
           ((mfd     & 0x7F) << 0))

/* Prepare value for PLL1 DV register */
#define UC_PLL1_DV_VAL(mfd, rfdphi) \
(uint32_t)((((rfdphi / 2) & 0x3F) << 16) |  \
           ((mfd & 0x7F)          << 0))

/* ----------------------------------------------------------------------------
 * SIUL2 module
 * --------------------------------------------------------------------------*/

/* Registers */
#define UC_SIUL_MCSR_IO_BASE      (vuint32_t*) (UC_SIUL_BASE + 0x0240) /* MCSR_IO[0] */
#define UC_SIUL_GPDO_BASE         (vuint8_t*)  (UC_SIUL_BASE + 0x1300) /* GPDO[0] */
#define UC_SIUL_GPDI_BASE         (vuint8_t*)  (UC_SIUL_BASE + 0x1500) /* GPDI[0] */

/* Configures pin in Input mode */
#define UC_SIUL_MSCR_IO_SET_INPUT(reg,wpde,wpue) \
(uint32_t)((reg) = (((reg) & 0x37B080FF) | \
                    ((0x1)        << 19) | \
                    ((wpde & 0x1) << 17) | \
                    ((wpue & 0x1) << 16)))

/* Configures pin in output mode and Clear input setting! */
#define UC_SIUL_MSCR_IO_SET_OUTPUT(reg,oerc,odc,sss) \
(uint32_t)((reg) = (((reg) & 0x00F00000) | \
                    ((oerc & 0x3) << 28) | \
                    ((odc & 0x7)  << 24) | \
                    ((sss & 0xff) << 0)))

/* ----------------------------------------------------------------------------
 * SWT module
 * --------------------------------------------------------------------------*/

/* Registers */
#define UC_SWT_CR_REG_OFFSET      (uint32_t) 0x00
#define UC_SWT_SR_REG_OFFSET      (uint32_t) 0x10

/* ----------------------------------------------------------------------------
 * PFLASH module
 * --------------------------------------------------------------------------*/

/* Registers */
#define UC_PFLASH_PFCR1_REG_OFFSET     (uint32_t) 0x00
#define UC_PFLASH_PFCR2_REG_OFFSET     (uint32_t) 0x04
#define UC_PFLASH_PFCR3_REG_OFFSET     (uint32_t) 0x08

/* By default enable Prefetch for data & instructions, enable buffers,
 * allow all masters configure apc, wait_States and prefetch scheme */
#define UC_PFLASH_PFCR1_VAL(apc, ws, pflim) \
(uint32_t)(((apc   & 0x07) << 13) | \
           ((ws    & 0x1F) <<  8) | \
           ((pflim & 0x03) <<  1) | \
           (0xFFFF0051))

/* By default enable all masters to Prefetch for data & instructions,
 * enable buffers */
#define UC_PFLASH_PFCR2_VAL(pflim) \
(uint32_t)(((pflim & 0x3) << 1) |  \
           (0xFFFF0051))

/* Configure PFLASH controller ports' cache way and arbitration
 * between ports */
#define UC_PFLASH_PFCR3_VAL(p0wcfg,p1wcfg,arbm) \
(uint32_t)(((p0wcfg & 0x3) << 30) | \
           ((p1wcfg & 0x3) << 28) | \
           ((arbm   & 0x3) << 14))

/* ----------------------------------------------------------------------------
 * PRAM module
 * --------------------------------------------------------------------------*/

/* Registers */
#define UC_PRAM_PRCR1_REG_OFFSET       (uint32_t) 0x00

/* Set PRAM PRCR1 register fields */
#define UC_PRAM_PRCR1_VAL(pri,p0rb,p1rb,ftdis) \
(uint32_t)(((pri   & 0x3) << 8) | \
           ((p1rb  & 0x1) << 5) | \
           ((p0rb  & 0x1) << 4) | \
           ((ftdis & 0x1) << 0))

/* ----------------------------------------------------------------------------
 * CORE module
 * --------------------------------------------------------------------------*/

/* Core SPR register numbers */
#define UC_CORE_DCACHE_SPR              1010
#define UC_CORE_ICACHE_SPR              1011

/* write to Core SPR register */
#define __mtspr(spr, value) \
	__asm__ volatile ("mtspr %0,%1" : : "n" (spr), "r" (value))

/* read from Core SPR register */
#define __mfspr(spr) \
  ({ unsigned long result; \
  __asm__ volatile ("mfspr %0,%1" : "=r" (result) : "n" (spr)); result; })

/* Context sync instructions */
#define __isync() __asm__ volatile ("isync" : : : "memory")
#define __sync()  __asm__ volatile ("sync" : : : "memory")
#define __msync() __asm__ volatile ("msync" : : : "memory")
#define __mbar()  __asm__ volatile ("mbar" : : : "memory")

/* ----------------------------------------------------------------------------
 * PIT0 module
 * Only PIT0 Channel 0 is supported by BSP
 * --------------------------------------------------------------------------*/

/* Registers */
#define UC_PIT_MCR_REG            *(vuint32_t*) (UC_PIT0_BASE + 0x0000)
#define UC_PIT_LDVAL_REG_ADDR      (uint32_t)   (UC_PIT0_BASE + 0x0100)
#define UC_PIT_CVAL_REG_ADDR       (uint32_t)   (UC_PIT0_BASE + 0x0104)
#define UC_PIT_TCTRL_REG_ADDR      (uint32_t)   (UC_PIT0_BASE + 0x0108)
#define UC_PIT_TFLG_REG_ADDR       (uint32_t)   (UC_PIT0_BASE + 0x010C)
#define UC_PIT_CHANNEL_OFFSET     0x10

/* pointer to channel register value */
#define UC_PIT_CHANNEL_REG(reg_addr, channel) \
*(vuint32_t*)(reg_addr + (UC_PIT_CHANNEL_OFFSET * channel))

/* Bits */
#define UC_PIT_MCR_FRZ_BIT        (uint32_t) 0x01
#define UC_PIT_MCR_MDIS_BIT       (uint32_t) 0x02
#define UC_PIT_TCTRL_TEN_BIT      (uint32_t) 0x01
#define UC_PIT_TCTRL_TIE_BIT      (uint32_t) 0x02
#define UC_PIT_TFLG_TIF_BIT       (uint32_t) 0x01

/* ----------------------------------------------------------------------------
 * INTC module
 * --------------------------------------------------------------------------*/

/* Registers */
#define UC_INTC_BCR_REG          *(vuint32_t*) (UC_INTC + 0x0000)
#define UC_INTC_CPR_REG_BASE      (vuint32_t*) (UC_INTC + 0x0010)
#define UC_INTC_IACKR_REG_BASE    (vuint32_t*) (UC_INTC + 0x0020)
#define UC_INTC_EOIR_REG_BASE     (vuint32_t*) (UC_INTC + 0x0030)
#define UC_INTC_PSR_REG_BASE      (vuint16_t*) (UC_INTC + 0x0060)

/* BCR value composition */
#define UC_INTC_BCR_VAL(core,mode) \
(uint32_t)(((uint32_t)(mode) & 0x01) << (core * 4))

/* PSR value composition */
#define UC_INTC_PSR_VAL(core,prio) \
(uint16_t)(((uint16_t)0x8000 >> (coreId & 0x3)) | \
           ((uint16_t)prio & 0x3F))


#endif /* UC_SPC58NN84_REGDEFS_H */

